// EMC_M5StickC_PulseOximeter Ver.20200517 by JO3EMC

// ライブラリ ヘッダーファイル読込
#include <Wire.h>
#include <MAX30100_PulseOximeter.h>
#include <M5StickC.h>
#include <BluetoothSerial.h>

// マクロ&グローバル変数定義
#define SPEED_SERIAL 115200 // シリアル通信のデータ転送レート 300 1200 2400 4800 9600 14400 19200 28800 38400 57600 115200
#define MAX30100_LED_CURR MAX30100_LED_CURR_24MA // MAX30100 LED電流 要調整 0 4_4 7_6 11 14_2 17_4 20_8 24 27_1 30_6 33_8 37 40_2 43_6 46_8 50
/*
#define MIN_SpO2         0        // 外れ値除外 下限 血中酸素濃度
#define MAX_SpO2      1000        // 外れ値除外 上限 血中酸素濃度
#define MIN_HeartRate    0        // 外れ値除外 下限 心拍数
#define MAX_HeartRate 1000        // 外れ値除外 上限 心拍数
#define N_SpO2      1             // 移動平均 集計数 血中酸素濃度
#define N_HeartRate 1             // 移動平均 集計数 心拍数
uint8_t a_SpO2[N_SpO2];           // 移動平均 履歴配列 血中酸素濃度
float   a_HeartRate[N_HeartRate]; // 移動平均 履歴配列 心拍数
int     i_SpO2      = 0;          // 移動平均 履歴配列現在地インデックス 血中酸素濃度
int     i_HeartRate = 0;          // 移動平均 履歴配列現在地インデックス 心拍数
*/

// インスタンス生成
BluetoothSerial SerialBT; // Bluetoothシリアル
PulseOximeter POM;        // パルスオキシメーター


void setup() {
  
  M5.begin();             // M5ライブラリ初期化
//  setCpuFrequencyMhz(80); // 消費電力を抑えるため、CPU周波数を抑制 10 20 40 80 160 240
  
  Serial.begin(SPEED_SERIAL,SERIAL_8N1); // シリアル通信のデータ転送レートを設定
  SerialBT.begin("M5StickC");            // Bluetoothシリアルのデバイス名を設定

  while (!POM.begin()) { // パルスオキシメーター初期化エラー処理
    Serial.println("Failed to begin PulseOximeter");
    M5.Lcd.setCursor(0,0,2);
    M5.Lcd.printf("Failed\nto\nbegin");
    delay(1000);
  } 
  ResetDisplay(0); // 画面リセット

  POM.setIRLedCurrent(MAX30100_LED_CURR); // MAX30100 LED電流を設定
  POM.setOnBeatDetectedCallback(Beat);    // 心拍検出時のコールバック関数設定
  
/*
  for (int i = 0; i < N_SpO2     ; i++) {a_SpO2[i]      = 0;}
  for (int i = 0; i < N_HeartRate; i++) {a_HeartRate[i] = 0;}
*/
  
}

void loop() {
  
  static unsigned long ms_old = 0;
  
  M5.update(); // M5ボタン状態更新
  if ( M5.BtnA.wasPressed() ) { // ボタンA(HOME)が押された場合
    ResetDisplay(1); // 画面リセット（回転）
  }
  
  POM.update(); // パルスオキシメーター状態更新
  unsigned long ms_new = millis();
  if ( ms_new - ms_old > 1000 ) {
    
    // パルスオキシメーター 値取得
    uint8_t SpO2 = POM.getSpO2();
    float   HeartRate = POM.getHeartRate();
//    uint8_t RedLedCurrentBias = POM.getRedLedCurrentBias();
    
/*
    // 外れ値除外&移動平均履歴記録処理
    if ( (MIN_SpO2 <= SpO2) && (SpO2 <= MAX_SpO2) ) {
      a_SpO2[i_SpO2] = SpO2;
      i_SpO2++;
      if (i_SpO2 >= N_SpO2) {i_SpO2 = 0;}
    }
    if ( (MIN_HeartRate <= HeartRate) && (HeartRate <= MAX_HeartRate) ) {
      a_HeartRate[i_HeartRate] = HeartRate;
      i_HeartRate++;
      if (i_HeartRate >= N_HeartRate) {i_HeartRate = 0;}
    }
    // 移動平均集計
    float ma_SpO2 = 0;
    float ma_HeartRate = 0;
    for (int i = 0; i < N_SpO2     ; i++) {ma_SpO2      += a_SpO2[i];}
    for (int i = 0; i < N_HeartRate; i++) {ma_HeartRate += a_HeartRate[i];}
    ma_SpO2      = ma_SpO2      / N_SpO2;
    ma_HeartRate = ma_HeartRate / N_HeartRate;
//    // シリアル出力 移動平均計算デバッグ用
//    Serial.println("SpO2\t"   +String(SpO2)   +"\tHeartRate\t"   +String(HeartRate));
//    Serial.println("ma_SpO2\t"+String(ma_SpO2)+"\tma_HeartRate\t"+String(ma_HeartRate));
//    Serial.printf("a_SpO2(%d)",i_SpO2);
//    for (int i = 0; i < N_SpO2     ; i++) {Serial.print("\t"+String(a_SpO2[i]));}
//    Serial.println();
//    Serial.printf("a_HeartRate(%d)",i_HeartRate);
//    for (int i = 0; i < N_HeartRate; i++) {Serial.print("\t"+String(a_HeartRate[i]));}
//    Serial.println();
    // 移動平均計算結果戻し代入
    SpO2 = (uint8_t) round(ma_SpO2);
    HeartRate = ma_HeartRate;
*/
    
    // シリアル出力 最終値
    String s = "SpO2(%)\t"+String(SpO2)+"\tHeartRate(bpm)\t"+String(HeartRate);
    Serial.println(s);   // USBシリアル出力
    SerialBT.println(s); // Bluetoothシリアル出力
    
    // LCD出力
    M5.Lcd.setTextFont(4);
    M5.Lcd.setTextColor(YELLOW,BLACK);
    uint8_t r = M5.Lcd.getRotation();
    if ( r == 0 || r == 2 ) { // 縦画面の場合
      M5.Lcd.setCursor(1,25);
      M5.Lcd.printf("%03d",SpO2);
      M5.Lcd.setCursor(1,70);
      M5.Lcd.printf("%03.0f",round(HeartRate));
    } else { // 横画面の場合
      M5.Lcd.setCursor(70,5);
      M5.Lcd.printf("%03d",SpO2);
      M5.Lcd.setCursor(70,30);
      M5.Lcd.printf("%03.0f",round(HeartRate));
    }
    printBeat(0);
    
    ms_old = ms_new;
    
  }
  
}


// 画面リセット
void ResetDisplay(char flag) {
  
  uint8_t r = M5.Lcd.getRotation();
  if (flag != 0) { // 引数が0でない場合は画面の向きを回転
    r++;
    if (r > 3) {r = 0;}
  }
  M5.Lcd.setRotation(r);     // 画面の向きを設定
//  M5.Axp.ScreenBreath(8);    // 消費電力を抑えるため、液晶バックライト電圧を暗く設定
  M5.Lcd.fillScreen(BLACK);  // 画面消去
  M5.Lcd.setTextWrap(0);     // テキストを折り返さない

  // 固定文字列出力
  M5.Lcd.setTextColor(WHITE,BLACK);
  if ( r == 0 || r == 2 ) { // 縦画面の場合
    M5.Lcd.setCursor(1,125,1);
    M5.Lcd.print("EMC_M5StickC");
    M5.Lcd.setTextFont(2);
    M5.Lcd.setCursor(1,132);
    M5.Lcd.print("Pulse");
    M5.Lcd.setCursor(1,145);
    M5.Lcd.print("Oximeter");
    M5.Lcd.setCursor(1,5);
    M5.Lcd.print("SpO2 (%)");
    M5.Lcd.setCursor(1,50);
    M5.Lcd.print("HR (bpm)");
    M5.Lcd.setCursor(1,95);
    M5.Lcd.print("Beat");
  } else { // 横画面の場合
    M5.Lcd.setCursor(2,72,1);
    M5.Lcd.print("EMC_M5StickC");
    M5.Lcd.setTextFont(2);
    M5.Lcd.setCursor(75,66);
    M5.Lcd.print("PulseOximeter");
    M5.Lcd.setCursor(5,7);
    M5.Lcd.print("SpO2 (%)");
    M5.Lcd.setCursor(5,32);
    M5.Lcd.print("HR (bpm)");
    M5.Lcd.setCursor(5,50);
    M5.Lcd.print("Beat");
  }
  
}

// 心拍検出時のコールバック関数
void Beat() { // ビート表示後、少し待って表示消去（数値測定間隔にブレが生じますが、本スケッチでは諦め）
  printBeat(1);
  delay(100);
  printBeat(0);
}
// 心拍表示
void printBeat(bool beat) {
  M5.Lcd.setTextFont(2);
  if (beat) { // ビート時
    M5.Lcd.setTextColor(RED,BLACK);
  } else { // 平時
    M5.Lcd.setTextColor(WHITE,BLACK);
  }
  uint8_t r = M5.Lcd.getRotation();
  if ( r == 0 || r == 2 ) { // 縦画面の場合
    M5.Lcd.setCursor(35,95);
  } else { // 横画面の場合
    M5.Lcd.setCursor(70,50);
  }
  M5.Lcd.print("@");
}
